'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var utils = require('./utils.js');
var globalSettings = require('./globalSettings.js');

var Binder = {
  bind: function($element) {
    var args = Array.prototype.slice.call(arguments, 1);
    attachCommands($element, args);
    var result = $element.find('[data-role^="telerik_ReportViewer_"]');
    utils.each(result, function() {
      var $this = $(this);
      var f = $.fn[$this.attr("data-role")];
      if (typeof f === "function") {
        f.apply($this, args);
      }
    });
  }
};
function attachCommands($element, args) {
  var commands = args[0].commands;
  var viewerOptions = args[1];
  var elementSelector = '[data-command^="telerik_ReportViewer_"]';
  var customElementSelector = "[data-target-report-viewer]" + elementSelector;
  $element.on("click", elementSelector, commandHandler);
  if (!globalSettings.GlobalSettings.CommandHandlerAttached) {
    $(document.body).on("click", customElementSelector, customCommandHandler);
    globalSettings.GlobalSettings.CommandHandlerAttached = true;
  }
  utils.each(commands, function(key, command) {
    attachCommand(key, command, viewerOptions, $element);
  });
  function commandHandler(e) {
    var prefixedDataCommand = $(this).attr("data-command");
    if (prefixedDataCommand) {
      var dataCommand = prefixedDataCommand.substring("telerik_ReportViewer_".length);
      var cmd = commands[dataCommand];
      if (cmd && cmd.enabled()) {
        cmd.exec($(this).attr("data-command-parameter"));
      }
      e.preventDefault();
    }
  }
  function customCommandHandler(e) {
    var $this = $(this);
    var prefixedDataCommand = $this.attr("data-command");
    var reportViewerTarget = $this.attr("data-target-report-viewer");
    if (prefixedDataCommand && reportViewerTarget) {
      var dataCommand = prefixedDataCommand.substring("telerik_ReportViewer_".length);
      var reportViewer = $(reportViewerTarget).data("telerik_ReportViewer");
      var cmd = reportViewer.commands[dataCommand];
      if (cmd.enabled()) {
        cmd.exec($(this).attr("data-command-parameter"));
      }
      e.preventDefault();
    }
  }
}
function attachCommand(dataCommand, cmd, viewerOptions, $element) {
  if (cmd) {
    var elementSelector = '[data-command="telerik_ReportViewer_' + dataCommand + '"]';
    var customElementSelector = '[data-target-report-viewer="' + viewerOptions.selector + '"]' + elementSelector;
    var $defaultElement = $element.find(elementSelector);
    var $customElement = $(customElementSelector);
    $(cmd).on("enabledChanged", function(e) {
      (cmd.enabled() ? $.fn.removeClass : $.fn.addClass).call($defaultElement.parent("li"), "k-disabled");
      (cmd.enabled() ? $.fn.removeClass : $.fn.addClass).call($customElement, viewerOptions.disabledButtonClass);
    }).on("checkedChanged", function(e) {
      var defaultTarget = $defaultElement.parent("li");
      (cmd.checked() ? $.fn.addClass : $.fn.removeClass).call(defaultTarget, getSelectedClassName(defaultTarget));
      (cmd.checked() ? $.fn.addClass : $.fn.removeClass).call($customElement, viewerOptions.checkedButtonClass);
    });
  }
}
function getSelectedClassName($element) {
  return $element.hasClass("trv-menu-toggleable") ? "k-selected !k-bg-primary" : "k-selected";
}
function LinkButton(dom, options) {
  var cmd;
  var $element = $(dom);
  var dataCommand = $element.attr("data-command");
  if (dataCommand) {
    cmd = options.commands[dataCommand];
  }
  if (cmd) {
    $element.click(function(e) {
      if (cmd.enabled()) {
        cmd.exec($(this).attr("data-command-parameter"));
      } else {
        e.preventDefault();
      }
    });
    $(cmd).on("enabledChanged", function(e) {
      (cmd.enabled() ? $.fn.removeClass : $.fn.addClass).call($element, "disabled");
    }).on("checkedChanged", function(e) {
      (cmd.checked() ? $.fn.addClass : $.fn.removeClass).call($element, "checked");
    });
  }
}
var linkButton_pluginName = "telerik_ReportViewer_LinkButton";
$.fn[linkButton_pluginName] = function(options) {
  return utils.each(this, function() {
    if (!$.data(this, linkButton_pluginName)) {
      $.data(this, linkButton_pluginName, new LinkButton(this, options));
    }
  });
};
function PageNumberInput(dom, options) {
  var $element = $(dom);
  var oldValue = 0;
  var cmd = options.commands["goToPage"];
  function setPageNumber(value) {
    if (oldValue !== value || !$element.is(":focus")) {
      $element.val(value);
      oldValue = value;
    }
  }
  options.controller.pageNumberChange(function(e, value) {
    setPageNumber(value);
  });
  $element.change(function() {
    var val = $(this).val();
    var num = utils.tryParseInt(val);
    if (!isNaN(num)) {
      var result = cmd.exec(num);
      setPageNumber(result);
    }
  });
  $element.keydown(function(e) {
    if (e.which == 13) {
      $(this).change();
      return e.preventDefault();
    }
  });
  function validateValue(value) {
    return /^([0-9]+)$/.test(value);
  }
  $element.keypress(function(event) {
    if (utils.isSpecialKey(event.keyCode)) {
      return true;
    }
    var newValue = $element.val() + String.fromCharCode(event.charCode);
    return validateValue(newValue);
  }).on("paste", function(event) {
  });
}
var pageNumberInput_pluginName = "telerik_ReportViewer_PageNumberInput";
$.fn[pageNumberInput_pluginName] = function(options) {
  return utils.each(this, function() {
    if (!$.data(this, pageNumberInput_pluginName)) {
      $.data(this, pageNumberInput_pluginName, new PageNumberInput(this, options));
    }
  });
};
function PageCountLabel(dom, options) {
  var $element = $(dom);
  options.controller.pageCountChange(function(e, value) {
    $element.text(value);
  });
}
var pageCountLabel_pluginName = "telerik_ReportViewer_PageCountLabel";
$.fn[pageCountLabel_pluginName] = function(options) {
  return utils.each(this, function() {
    if (!$.data(this, pageCountLabel_pluginName)) {
      $.data(this, pageCountLabel_pluginName, new PageCountLabel(this, options));
    }
  });
};

exports.Binder = Binder;
