import { each, stringFormat, findElement } from './utils.js';
import { stringResources } from './stringResources.js';

function SideMenu(dom, rootOptions, otherOptions) {
  var options = $.extend({}, rootOptions, otherOptions);
  var menu = $(dom).data("kendoMenu");
  var enableAccessibility = options.enableAccessibility;
  var DEFAULT_TABINDEX = 3;
  var panelBar;
  var sideMenuVisible = false;
  var controller = options.controller;
  if (!controller) {
    throw "No controller (telerikReporting.ReportViewerController) has been specified.";
  }
  if (!menu) {
    init(dom);
  }
  function init(root) {
    var $root = $(root);
    try {
      panelBar = $root.children("ul").kendoPanelBar().data("kendoPanelBar");
    } catch (e) {
      console.error("Instantiation of Kendo PanelBar as side menu threw an exception", e);
      throw e;
    }
    panelBar.bind("expand", onSubmenuOpen);
    panelBar.element.off("keydown", onPanelKeyDown);
    panelBar.element.on("keydown", onPanelKeyDown);
    setTabIndexes($root);
    enableCloseOnClick($root);
    $root.click(function(e) {
      if (e.target == root) {
        controller.setSideMenuVisible({ visible: !sideMenuVisible });
      }
    });
    replaceStringResources();
  }
  controller.setSideMenuVisible(function(event, args) {
    setSideMenuVisibility();
    if (enableAccessibility) {
      panelBar.element.focus();
    }
    sideMenuVisible = args.visible;
    if (!sideMenuVisible) {
      panelBar.collapse($("#trv-side-menu-export-command"));
    }
  }).getSideMenuVisible(function(event, args) {
    args.visible = sideMenuVisible;
  });
  function setSideMenuVisibility() {
    var $root = panelBar.element.parent();
    var hidden = $root.position().left < 0 || !$root.is(":visible");
    if (hidden) {
      $root.show();
    } else {
      window.setTimeout(function() {
        $root.hide();
      }, 500);
    }
  }
  function onSubmenuOpen(e) {
    var $item = $(e.item);
    panelBar.unbind("expand", onSubmenuOpen);
    panelBar.append({ text: stringResources.loadingFormats, spriteCssClass: "k-icon k-loading" }, $item);
    options.controller.getDocumentFormats().then(fillFormats).then(function() {
      panelBar.expand($item);
    }).then(function() {
      panelBar.bind("expand", onSubmenuOpen);
    });
  }
  function fillFormats(formats) {
    each($(dom).find("ul[data-command-list=export-format-list]"), function() {
      var $list = $(this);
      var $parent = $list.parents("li");
      var tabIndex = $parent.attr("tabindex");
      if (!tabIndex) {
        tabIndex = DEFAULT_TABINDEX;
      }
      $list.empty();
      each(formats, function(i) {
        var format = this;
        var ariaLabel = enableAccessibility ? stringFormat('aria-label="{localizedName}" ', format) : " ";
        var li = "<li " + ariaLabel + stringFormat('tabindex="' + tabIndex + '"><a tabindex="-1" href="#" data-command="telerik_ReportViewer_export" data-command-parameter="{name}"><span>{localizedName}</span></a></li>', format);
        panelBar.append(li, $parent);
      });
      setListItemsTabIndex($parent.find("li"), tabIndex);
      enableCloseOnClick($parent);
    });
  }
  function enableCloseOnClick(root) {
    each(root.find("li"), function() {
      var isLeaf = $(this).children("ul").length === 0;
      if (isLeaf) {
        $(this).children("a").click(function() {
          controller.setSideMenuVisible({ visible: !sideMenuVisible });
        });
      }
    });
  }
  function setTabIndexes(root) {
    if (!root) {
      return;
    }
    var $list = root.children("ul");
    var parentTabIndex = root.attr("tabindex");
    var listIndex = parentTabIndex ? parentTabIndex : DEFAULT_TABINDEX;
    setListItemsTabIndex($list, listIndex);
  }
  function setListItemsTabIndex(list, tabIndex) {
    list.attr("tabindex", tabIndex);
    var items = list.find("li");
    each(items, function() {
      var $item = $(this);
      $item.attr("tabindex", tabIndex);
      var anchor = $item.children("a");
      if (anchor.length > 0) {
        var $anchor = $(anchor);
        $anchor.attr("tabindex", -1);
      }
      $item.focus(function() {
        var anchor2 = $item.children("a");
        if (anchor2.length > 0) {
          anchor2.addClass("k-focus");
        }
      });
      $item.blur(function() {
        var anchor2 = $item.children("a");
        if (anchor2.length > 0) {
          anchor2.removeClass("k-focus");
        }
      });
      $item.off("keydown", onItemKeyDown);
      $item.on("keydown", onItemKeyDown);
    });
  }
  function onPanelKeyDown(e) {
    if (e.which == kendo.keys.ENTER) {
      var $item;
      var isSelectedFocusedItem = false;
      var focusedItem = document.activeElement;
      if (focusedItem && focusedItem.localName == "li") {
        var items = panelBar.element.find("li.k-item");
        for (var i = 0; i < items.length; i++) {
          var listItem = items[i];
          if (focusedItem === listItem) {
            $item = $(listItem);
            isSelectedFocusedItem = true;
            break;
          }
        }
      } else {
        $item = panelBar.select();
      }
      if (!$item || !$item.length > 0) {
        return;
      }
      handleItemSelect($item, isSelectedFocusedItem);
    }
  }
  function onItemKeyDown(e) {
    if (e.which == kendo.keys.ENTER) {
      handleItemSelect($(e.target), false);
    }
  }
  function handleItemSelect(item, handleExpandCollapse) {
    if (!item.length > 0) {
      return;
    }
    var isLeaf = item.children("ul").length === 0;
    if (!isLeaf) {
      if (handleExpandCollapse) {
        if (item.hasClass("k-active")) {
          panelBar.collapse(item);
        } else {
          panelBar.expand(item);
        }
      }
    } else {
      var $anchor = item.find("a");
      if ($anchor.length > 0) {
        $anchor[0].click();
      }
    }
  }
  function replaceStringResources() {
    var menuAreas = findMenuArea();
    if (!menuAreas) {
      return;
    }
    each(menuAreas, function() {
      var $menu = $(this);
      var menuItems = $menu.children("li.k-panelbar-header");
      $menu.attr("aria-label", stringResources[$menu.attr("aria-label")]);
      each(menuItems, function() {
        var $menuItem = $(this);
        var $a = $menuItem.find("a");
        $menuItem.attr("aria-label", stringResources[$menuItem.attr("aria-label")]);
        if ($a) {
          var $span = $a.find("span:not(.k-icon)");
          $a.attr("title", stringResources[$a.attr("title")]);
          if ($span) {
            $span.text(stringResources[$span.text()]);
          }
        }
      });
    });
  }
  function findMenuArea() {
    return findElement("div[data-role=telerik_ReportViewer_SideMenu] > ul");
  }
}
var pluginName = "telerik_ReportViewer_SideMenu";
$.fn[pluginName] = function(options, otherOptions) {
  return each(this, function() {
    if (!$.data(this, pluginName)) {
      $.data(this, pluginName, new SideMenu(this, options, otherOptions));
    }
  });
};

export { SideMenu };
